-------------------------------------------------------------------------------
--
-- Simple VGA raster display
-- Team	NAHVG
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity de2_vga_raster is
  
  port (
    reset_n : in std_logic;
    clk   : in std_logic;                   
--------------------------------------------------------------------------------
    
    read : in std_logic;
    write : in std_logic;
    chipselect : in std_logic;
    address : in unsigned(4 downto 0);
    readdata : out unsigned(15 downto 0);
    writedata : in unsigned(15 downto 0);

    VGA_CLK,                         -- Clock		 -- Should be 25.125 MHz
    VGA_HS,                          -- H_SYNC
    VGA_VS,                          -- V_SYNC
    VGA_BLANK,                       -- BLANK
    VGA_SYNC : out std_logic;        -- SYNC
    VGA_R,                           -- Red[9:0]
    VGA_G,                           -- Green[9:0]
    VGA_B : out unsigned(9 downto 0) -- Blue[9:0]
    );

end de2_vga_raster;

architecture rtl of de2_vga_raster is
  
  -- Video parameters
  
  constant HTOTAL       : integer := 800;
  constant HSYNC        : integer := 96;
  constant HBACK_PORCH  : integer := 48;
  constant HACTIVE      : integer := 640;
  constant HFRONT_PORCH : integer := 16;
  
  constant VTOTAL       : integer := 525;
  constant VSYNC        : integer := 2;
  constant VBACK_PORCH  : integer := 33;
  constant VACTIVE      : integer := 480;
  constant VFRONT_PORCH : integer := 10;

  constant RECTANGLE_HSTART : integer := 100;
  constant RECTANGLE_HEND   : integer := 540;
  constant RECTANGLE_VSTART : integer := 100;
  constant RECTANGLE_VEND   : integer := 380;

  -- Signals for the video controller
  signal Hcount : unsigned(9 downto 0);  -- Horizontal position (0-800)
  signal Vcount : unsigned(9 downto 0);  -- Vertical position (0-524)
  signal EndOfLine, EndOfField : std_logic;
 
  signal vga_hblank, vga_hsync,
  vga_vblank, vga_vsync : std_logic;  -- Sync. signals

  constant goal_upper_slot		:  integer := 252;
  constant goal_lower_slot		:  integer := 212;
  signal user_cxstart			:  unsigned(9 downto 0) := "1000001101";			--525
  signal user_cystart			:  unsigned(9 downto 0) := "0001110000";			--112
  signal puck_cxstart			:  unsigned(9 downto 0) := "0101000000";			--320
  signal puck_cystart           :  unsigned(9 downto 0) := "0011101000"; 			--232
  constant radius	            	:  integer := 10;
  constant centre_radius			:  integer := 20;
  constant small_center_radius  	:  integer := 18;
  constant gradius_inner			:  integer := 18;
  constant gradius 					:  integer := 20;

  constant sqr_radius	            :  integer := 100;
  constant sqr_centre_radius		:  integer := 400;
  constant sqr_small_center_radius	:  integer := 324;
  constant sqr_gradius_inner		:  integer := 324;
  constant sqr_gradius 				:  integer := 400;

  signal ethernet_cxstart		:  unsigned(9 downto 0) := "0001111111";			--127
  signal ethernet_cystart		:  unsigned(9 downto 0) := "0001111111";			--127

  constant center_cxstart		:  unsigned(9 downto 0) := "0101000000";			--320
  constant center_cystart		:  unsigned(9 downto 0) := "0011101000";			--232
  constant center_gxstart_l		:  unsigned(9 downto 0) := "0000000000";			--0
  constant center_gxstart_r		:  unsigned(9 downto 0) := "1010000000";			--640
  constant center_gystart		:  unsigned(9 downto 0) := "0011101000";			--232
  signal vline,goal,goal_outer_circle, goal_inner_circle,
		circle,ethernet_circle,center_circle,small_center_circle,puck_circle,
		score_player1,score_player2, sc_player_1, sc_player_2 : std_logic := '0';
		
  signal vline_off				:  integer := 0;
  signal clk25 : std_logic :='0';
  signal reset : std_logic;
  signal tmp : unsigned (9 downto 0) := (others => '0');

  signal player_1 , player_2 : integer :=0;

  type player1 is array ( 0 to 7) of unsigned ( 0 to 50);
  constant crange : player1 := (
"011110001000000011110001000101111100111111001100000",
"010001001000000100001001000101000000100001001100000",
"010001001000000100001001000101000000100001001100110",
"011110001000000100001000111001000000111111001100000",
"010000001000000111111000010001110000110000001100000",
"010000001000000100001000010001000000101000001100110",
"010000001000000100001000010001000000100100001100000",
"010000001111100100001000010001111100100011001100000");

type player2 is array ( 0 to 7) of unsigned ( 0 to 54);
constant crange1 : player2 := (
"0111100010000000111100010001001111100111111001111100000",
"0100010010000001000010010001001000000100001000001100000",
"0100010010000001000010010001001000000100001000001100110",
"0111100010000001000010001110001000000111111001111100000",
"0100000010000001111110000100001110000110000001100000000",
"0100000010000001000010000100001000000101000001100000110",
"0100000010000001000010000100001000000100100001100000000",
"0100000011111001000010000100001111100100011001111100000");

type scoredecode is array ( 0 to 7) of unsigned ( 0 to 71);
constant crange2 : scoredecode := (
"001111100010000001111100011111000100010001111100011111000111110001111100",
"001000100010000000000100000001000100010001000000010000000000010001000100",
"001000100010000000000100000001000100010001000000010000000000010001000100",
"001000100010000001111100000001000111110001111100010000000000100001000100",
"001000100010000001000000011111000000010000000100011111000001000001111100",
"001000100010000001000000000001000000010000000100010001000001000001000100",
"001000100010000001000000000001000000010000000100010001000001000001000100",
"001111100010000001111100011111000000010001111100011111000001000001111100" );

-- Lookup table for squares of integers 0 to 20
 type mem is array (0 to 20) of integer;
 constant table : mem := ( 0,1,4,9,16,25,36,49,64,81,100,121,144,169,196,225,256,289,324,361,400);

begin

reset <= not reset_n;

--------------------------------Peripheral------------------------------------------------------
process (clk) 
  begin
    if rising_edge(clk) then
      clk25 <= not clk25;
    end if;
  end process;

process(clk)
  begin
if rising_edge(clk) then    --Make sure every logic is on rising edge of clock
 if reset_n='1' then		--To synchronise it
  if chipselect = '1' then
    if write ='1' then
      if address = "00000" then
       user_cxstart <= writedata(9 downto 0);
      elsif address = "00001" then
       user_cystart <= writedata(9 downto 0);
	  elsif address = "00010" then
       ethernet_cxstart <= writedata(9 downto 0);
      elsif address = "00011" then
       ethernet_cystart <= writedata(9 downto 0);
      elsif address = "00100" then
       puck_cxstart <= writedata(9 downto 0);
      elsif address = "00101" then
      puck_cystart <= writedata(9 downto 0);       
      elsif address <= "00111" then
      player_1 <=to_integer(writedata(9 downto 0));
      elsif address <= "01000" then
      player_2 <=to_integer(writedata(9 downto 0));
	  end if;
    end if;
  end if;
 end if;
end if;
end process;

 

  -- Horizontal and vertical counters

  HCounter : process (clk25)
  begin
    if rising_edge(clk25) then      
      if reset = '1' then
        Hcount <= (others => '0');
      elsif EndOfLine = '1' then
        Hcount <= (others => '0');
      else
        Hcount <= Hcount + 1;
      end if;      
    end if;
  end process HCounter;

  EndOfLine <= '1' when Hcount = HTOTAL - 1 else '0';
  
  VCounter: process (clk25)
  begin
    if rising_edge(clk25) then      
      if reset = '1' then
        Vcount <= (others => '0');
      elsif EndOfLine = '1' then
        if EndOfField = '1' then
          Vcount <= (others => '0');
        else
          Vcount <= Vcount + 1;
        end if;
      end if;
    end if;
  end process VCounter;

  EndOfField <= '1' when Vcount = VTOTAL - 1 else '0';

  -- State machines to generate HSYNC, VSYNC, HBLANK, and VBLANK

  HSyncGen : process (clk25)
  begin
    if rising_edge(clk25) then     
      if reset = '1' or EndOfLine = '1' then
        vga_hsync <= '1';
      elsif Hcount = HSYNC - 1 then
        vga_hsync <= '0';
      end if;
    end if;
  end process HSyncGen;
  
  HBlankGen : process (clk25)
  begin
    if rising_edge(clk25) then
      if reset = '1' then
        vga_hblank <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH then
        vga_hblank <= '0';
      elsif Hcount = HSYNC + HBACK_PORCH + HACTIVE then
        vga_hblank <= '1';
      end if;      
    end if;
  end process HBlankGen;

  VSyncGen : process (clk25)
  begin
    if rising_edge(clk25) then
      if reset = '1' then
        vga_vsync <= '1';
      elsif EndOfLine ='1' then
        if EndOfField = '1' then
          vga_vsync <= '1';
        elsif Vcount = VSYNC - 1 then
          vga_vsync <= '0';
        end if;
      end if;      
    end if;
  end process VSyncGen;

  VBlankGen : process (clk25)
  begin
    if rising_edge(clk25) then    
      if reset = '1' then
        vga_vblank <= '1';
      elsif EndOfLine = '1' then
        if Vcount = VSYNC + VBACK_PORCH - 1 then
          vga_vblank <= '0';
        elsif Vcount = VSYNC + VBACK_PORCH + VACTIVE - 1 then
          vga_vblank <= '1';
        end if;
      end if;
    end if;
  end process VBlankGen;
---------------------------------------Vertical  Line gen code--------------------------------
  Linegen : process(clk25)
  begin
	if rising_edge (clk25) then
		if reset = '1' then
			vline <= '0';
		elsif	((140 = Hcount-HBACK_PORCH-HSYNC)or(500 = Hcount-HBACK_PORCH-HSYNC)or(320 = Hcount-HBACK_PORCH-HSYNC))
				and Vcount<=464+VSYNC+VBACK_PORCH then
			
				vline <= '1';
		else
			vline <= '0';
		end if;
	end if;
 end process;

----------------------------------------------------------------------------------------------  
--------------------------------------User Control--------------------------------------------
  CircleGen1 : process(clk25)
  begin
  if rising_edge(clk25) then
		if reset = '1' then
			circle <= '0';
		elsif Hcount>= user_cxstart+HBACK_PORCH+HSYNC and Vcount >= user_cystart+VBACK_PORCH+VSYNC 
			and (Hcount-user_cxstart-HBACK_PORCH-HSYNC)<=radius and (Vcount-user_cystart-VBACK_PORCH-VSYNC) <=radius then --4th quadrant
			if ((table(TO_INTEGER(Hcount-user_cxstart)-HBACK_PORCH-HSYNC) + table(TO_INTEGER(Vcount-user_cystart)-VBACK_PORCH-VSYNC))	<= sqr_radius ) then
				circle <= '1';
			 else
				circle <= '0';
			 end if;
		elsif Hcount>= user_cxstart+HBACK_PORCH+HSYNC and Vcount <= user_cystart+VBACK_PORCH+VSYNC 
				and (Hcount-user_cxstart-HBACK_PORCH-HSYNC)<=radius and (user_cystart-Vcount+VBACK_PORCH+VSYNC) <=radius then  --1st quadrant
			if ((table(TO_INTEGER(Hcount-user_cxstart)-HBACK_PORCH-HSYNC)  + table(TO_INTEGER(user_cystart-Vcount)+VBACK_PORCH+VSYNC))<= sqr_radius) then
				circle <= '1';
			 else
				circle <= '0';
			 end if;
		elsif Hcount<= user_cxstart+HBACK_PORCH+HSYNC	and Vcount <= user_cystart+VBACK_PORCH+VSYNC 
			and (user_cxstart-Hcount+HBACK_PORCH+HSYNC)<=radius and (user_cystart-Vcount+VBACK_PORCH+VSYNC) <=radius then		--2nd quadrant
			if ((table(TO_INTEGER(user_cxstart-Hcount)+HBACK_PORCH+HSYNC)  + table(TO_INTEGER(user_cystart-Vcount)+VBACK_PORCH+VSYNC))	<= sqr_radius) then
				circle <= '1';
			 else
				circle<='0';
			 end if;
		elsif Hcount<= user_cxstart+HBACK_PORCH+HSYNC	and Vcount >= user_cystart+VBACK_PORCH+VSYNC
			and (user_cxstart-Hcount+HBACK_PORCH+HSYNC)<=radius and (Vcount-user_cystart-VBACK_PORCH-VSYNC) <=radius then	 --3rd quadrant
			if ((table(TO_INTEGER(user_cxstart-Hcount)+HBACK_PORCH+HSYNC)  + table(TO_INTEGER(Vcount-user_cystart)-VBACK_PORCH-VSYNC)) <= sqr_radius) then
				circle <= '1';
			 else
				circle<='0';
			 end if;
		else
				circle <= '0';
		end if;
	end if;
  end process;

-------------------------------------------------------------------------------------------------
  
---------------------------------------Second User Control(Ethernet)-----------------------------

  CircleGen2 : process(clk25)
  begin
  if rising_edge(clk25) then
		if reset = '1' then
			ethernet_circle <= '0';
		elsif Hcount>= ethernet_cxstart+HBACK_PORCH+HSYNC and Vcount >= ethernet_cystart+VBACK_PORCH+VSYNC 
			and (Hcount-ethernet_cxstart-HBACK_PORCH-HSYNC)<=radius and (Vcount-ethernet_cystart-VBACK_PORCH-VSYNC) <=radius then --4th quadrant
			if (table(TO_INTEGER(Hcount-ethernet_cxstart)-HBACK_PORCH-HSYNC)+ table(TO_INTEGER(Vcount-ethernet_cystart)-VBACK_PORCH-VSYNC) <= sqr_radius) then
				ethernet_circle <= '1';
			 else
				ethernet_circle <= '0';
			 end if;
		elsif Hcount>= ethernet_cxstart+HBACK_PORCH+HSYNC and Vcount <= ethernet_cystart+VBACK_PORCH+VSYNC 
			and (Hcount-ethernet_cxstart-HBACK_PORCH-HSYNC)<=radius and (ethernet_cystart+VBACK_PORCH+VSYNC-Vcount) <=radius then  --1st quadrant
			if (table(TO_INTEGER(Hcount-ethernet_cxstart)-HBACK_PORCH-HSYNC) + table(TO_INTEGER(ethernet_cystart-Vcount)+VBACK_PORCH+VSYNC)	<= sqr_radius) then
				ethernet_circle <= '1';
			 else
				ethernet_circle <= '0';
			 end if;
		elsif Hcount<= ethernet_cxstart+HBACK_PORCH+HSYNC	and Vcount <= ethernet_cystart+VBACK_PORCH+VSYNC 
			and (ethernet_cxstart+HBACK_PORCH+HSYNC-Hcount)<=radius and (ethernet_cystart+VBACK_PORCH+VSYNC-Vcount) <=radius then		--2nd quadrant
			if (table(TO_INTEGER(ethernet_cxstart-Hcount)+HBACK_PORCH+HSYNC)  + table(TO_INTEGER(ethernet_cystart-Vcount)+VBACK_PORCH+VSYNC) 	<= sqr_radius) then
				ethernet_circle <= '1';
			 else
				ethernet_circle<='0';
			 end if;
		elsif Hcount<= ethernet_cxstart+HBACK_PORCH+HSYNC	and Vcount >= ethernet_cystart+VBACK_PORCH+VSYNC 
			and (ethernet_cxstart+HBACK_PORCH+HSYNC-Hcount)<=radius and (Vcount-ethernet_cystart-VBACK_PORCH-VSYNC) <=radius then	 --3rd quadrant
			if (table(TO_INTEGER(ethernet_cxstart-Hcount)+HBACK_PORCH+HSYNC)+table(TO_INTEGER(Vcount-ethernet_cystart)-VBACK_PORCH-VSYNC) <= sqr_radius) then
				ethernet_circle <= '1';
			 else
				ethernet_circle<='0';
			 end if;
		else
				ethernet_circle <= '0';
		end if;
	end if;
  end process;

--------------------------------------------------------------------------------------------------
------------------------------center circle generation--------------------------------------------

  CircleGen3 : process(clk25)
  begin
  if rising_edge(clk25) then
		if reset = '1' then
			center_circle <= '0';
		elsif Hcount>= center_cxstart+HBACK_PORCH+HSYNC and Vcount >= center_cystart+VBACK_PORCH+VSYNC then --4th quadrant
			if (Hcount-center_cxstart-HBACK_PORCH-HSYNC) <= centre_radius and (Vcount-center_cystart-VBACK_PORCH-VSYNC) <=centre_radius 
				and (table(TO_INTEGER(Hcount-center_cxstart)-HBACK_PORCH-HSYNC) +table(TO_INTEGER(Vcount-center_cystart)-VBACK_PORCH-VSYNC)<= sqr_centre_radius) then
				center_circle <= '1';
			 else
				center_circle <= '0';
			 end if;
		elsif Hcount>= center_cxstart+HBACK_PORCH+HSYNC and Vcount <= center_cystart+VBACK_PORCH+VSYNC then  --1st quadrant
			if (Hcount-center_cxstart-HBACK_PORCH-HSYNC) <= centre_radius and (center_cystart+VBACK_PORCH+VSYNC-Vcount) <=centre_radius 
				and (table(TO_INTEGER(Hcount-center_cxstart)-HBACK_PORCH-HSYNC) +table(TO_INTEGER(center_cystart-Vcount)+VBACK_PORCH+VSYNC) <= sqr_centre_radius) then
				center_circle <= '1';
			 else
				center_circle <= '0';
			 end if;
		elsif Hcount<= center_cxstart+HBACK_PORCH+HSYNC	and Vcount <= center_cystart+VBACK_PORCH+VSYNC then		--2nd quadrant
			if (center_cxstart+HBACK_PORCH+HSYNC-Hcount) <= centre_radius and (center_cystart+VBACK_PORCH+VSYNC-Vcount) <=centre_radius 
				and (table(TO_INTEGER(center_cxstart-Hcount)+HBACK_PORCH+HSYNC) + table(TO_INTEGER(center_cystart-Vcount)+VBACK_PORCH+VSYNC) <= sqr_centre_radius) then	
				center_circle <= '1';
			 else
				center_circle<='0';
			 end if;
		elsif Hcount<= center_cxstart+HBACK_PORCH+HSYNC	and Vcount >= center_cystart+VBACK_PORCH+VSYNC   then	 --3rd quadrant
			if (center_cxstart+HBACK_PORCH+HSYNC-Hcount) <= centre_radius and (Vcount-center_cystart-VBACK_PORCH-VSYNC) <=centre_radius
				and (table(TO_INTEGER(center_cxstart-Hcount)+HBACK_PORCH+HSYNC) + table(TO_INTEGER(Vcount-center_cystart)-VBACK_PORCH-VSYNC) <= sqr_centre_radius) then	
				center_circle <= '1';
			 else
				center_circle<='0';
			 end if;
		else
				center_circle <= '0';
		end if;
	end if;
  end process;
---------------------------------------------------------------------------------------------------

---------------------------------------small center circle generation------------------------------
CircleGen4 : process(clk25)
  begin
  if rising_edge(clk25) then
		if reset = '1' then
			small_center_circle <= '0';
		elsif Hcount>= center_cxstart+HBACK_PORCH+HSYNC and Vcount >= center_cystart+VBACK_PORCH+VSYNC then --4th quadrant
			if (Hcount-center_cxstart-HBACK_PORCH-HSYNC) <= small_center_radius and (Vcount-center_cystart-VBACK_PORCH-VSYNC) <=small_center_radius 
				and (table(TO_INTEGER(Hcount-center_cxstart)-HBACK_PORCH-HSYNC)+table(TO_INTEGER(Vcount-center_cystart)-VBACK_PORCH-VSYNC)<= sqr_small_center_radius) then
				small_center_circle <= '1';
			 else
				small_center_circle <= '0';
			 end if;
		elsif Hcount>= center_cxstart+HBACK_PORCH+HSYNC and Vcount <= center_cystart+VBACK_PORCH+VSYNC  then  --1st quadrant
			if (Hcount-center_cxstart-HBACK_PORCH-HSYNC) <= small_center_radius and (center_cystart+VBACK_PORCH+VSYNC-Vcount) <=small_center_radius
				and (table(TO_INTEGER(Hcount-center_cxstart)-HBACK_PORCH-HSYNC)+table(TO_INTEGER(center_cystart-Vcount)+VBACK_PORCH+VSYNC) <= sqr_small_center_radius) then
				small_center_circle <= '1';
			 else
				small_center_circle <= '0';
			 end if;
		elsif Hcount<= center_cxstart+HBACK_PORCH+HSYNC	and Vcount <= center_cystart+VBACK_PORCH+VSYNC then		--2nd quadrant
			if (center_cxstart+HBACK_PORCH+HSYNC-Hcount)<= small_center_radius and (center_cystart+VBACK_PORCH+VSYNC-Vcount) <=small_center_radius 
				and (table(TO_INTEGER(center_cxstart-Hcount)+HBACK_PORCH+HSYNC)+table(TO_INTEGER(center_cystart-Vcount)+VBACK_PORCH+VSYNC) <= sqr_small_center_radius) then
				small_center_circle <= '1';
			 else
				small_center_circle<='0';
			 end if;
		elsif Hcount<= center_cxstart+HBACK_PORCH+HSYNC	and Vcount >= center_cystart+VBACK_PORCH+VSYNC then	 --3rd quadrant
			if (center_cxstart+HBACK_PORCH+HSYNC-Hcount)<= small_center_radius and (Vcount-center_cystart-VBACK_PORCH-VSYNC) <=small_center_radius 
				and (table(TO_INTEGER(center_cxstart-Hcount)+HBACK_PORCH+HSYNC)+table(TO_INTEGER(Vcount-center_cystart)-VBACK_PORCH-VSYNC) <= sqr_small_center_radius) then
				small_center_circle <= '1';
			 else
				small_center_circle<='0';
			 end if;
		else
				small_center_circle <= '0';
		end if;
	end if;
  end process;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------puck gen code-----------------------------------
  CircleGen5 : process(clk25)
  begin
  if rising_edge(clk25) then
		if reset = '1' then
			puck_circle <= '0';
		elsif Hcount>= puck_cxstart+HBACK_PORCH+HSYNC and Vcount >= puck_cystart+VBACK_PORCH+VSYNC then --4th quadrant
			if (Hcount-puck_cxstart-HBACK_PORCH-HSYNC)<= radius and (Vcount-puck_cystart-VBACK_PORCH-VSYNC)<=radius 
				and (table(TO_INTEGER(Hcount-puck_cxstart)-HBACK_PORCH-HSYNC)+table(TO_INTEGER(Vcount-puck_cystart)-VBACK_PORCH-VSYNC))<= sqr_radius then
				puck_circle <= '1';
			 else
				puck_circle <= '0';
			 end if;
		elsif Hcount>= puck_cxstart+HBACK_PORCH+HSYNC and Vcount <= puck_cystart+VBACK_PORCH+VSYNC then  --1st quadrant
			if (Hcount-puck_cxstart-HBACK_PORCH-HSYNC)<= radius and (puck_cystart+VBACK_PORCH+VSYNC-Vcount)<=radius 
				and (table(TO_INTEGER(Hcount-puck_cxstart)-HBACK_PORCH-HSYNC)+table(TO_INTEGER(puck_cystart-Vcount)+VBACK_PORCH+VSYNC))<= sqr_radius then
				puck_circle <= '1';
			 else
				puck_circle <= '0';
			 end if;
		elsif Hcount<= puck_cxstart+HBACK_PORCH+HSYNC	and Vcount <= puck_cystart+VBACK_PORCH+VSYNC then		--2nd quadrant
			if (puck_cxstart+HBACK_PORCH+HSYNC-Hcount)<= radius and (puck_cystart+VBACK_PORCH+VSYNC-Vcount)<=radius 
				and (table(TO_INTEGER(puck_cxstart-Hcount)+HBACK_PORCH+HSYNC)+table(TO_INTEGER(puck_cystart-Vcount)+VBACK_PORCH+VSYNC))<= sqr_radius then
				puck_circle <= '1';
			 else
				puck_circle<='0';
			 end if;
		elsif Hcount<= puck_cxstart+HBACK_PORCH+HSYNC	and Vcount >= puck_cystart+VBACK_PORCH+VSYNC then	 --3rd quadrant
			if (puck_cxstart+HBACK_PORCH+HSYNC-Hcount)<= radius and (Vcount-puck_cystart-VBACK_PORCH-VSYNC)<=radius
			and (table(TO_INTEGER(puck_cxstart-Hcount)+HBACK_PORCH+HSYNC)+table(TO_INTEGER(Vcount-puck_cystart)-VBACK_PORCH-VSYNC))<= sqr_radius then
				puck_circle <= '1';
			 else
				puck_circle<='0';
			 end if;
		else
				puck_circle <= '0';
		end if;
	end if;
  end process;

---------------------------------------------------------------------------------------------------
---------------------------------------------Goal Gen Code-----------------------------------------  
  Goalgen : process(clk25)
  begin
	if rising_edge (clk25) then
		if reset = '1' then
			goal_outer_circle <= '0';
		---------------------------Left Goal-----------------------
		elsif	(Hcount>= center_gxstart_l + HBACK_PORCH+HSYNC and Vcount >= center_gystart+VBACK_PORCH+VSYNC
					and Hcount <= center_gxstart_l + gradius + HBACK_PORCH + HSYNC) then					-- to take it to next condition
			   
			if (Hcount-center_gxstart_l - HBACK_PORCH - HSYNC)<=gradius and (Vcount-center_gystart-VBACK_PORCH-VSYNC) <= gradius
			and (table(TO_INTEGER(Hcount-center_gxstart_l )- HBACK_PORCH - HSYNC) + table(TO_INTEGER(Vcount-center_gystart)-VBACK_PORCH-VSYNC)) <= (sqr_gradius) then
				goal_outer_circle <= '1';
			else
				goal_outer_circle <= '0';
			end if;
	
		elsif	(Hcount>= center_gxstart_l + HBACK_PORCH + HSYNC and Vcount <= center_gystart + VBACK_PORCH + VSYNC
					and Hcount <= center_gxstart_l + gradius + HBACK_PORCH + HSYNC) then  
				
			if (Hcount-center_gxstart_l - HBACK_PORCH - HSYNC)<=gradius and (center_gystart-Vcount+VBACK_PORCH+VSYNC) <= gradius
			and (table(TO_INTEGER(Hcount-center_gxstart_l )- HBACK_PORCH - HSYNC) + table(TO_INTEGER(center_gystart-Vcount)+VBACK_PORCH+VSYNC)) <= (sqr_gradius) then
					goal_outer_circle <= '1'; 
			else
					goal_outer_circle <= '0';
			end if;
		
	-------------------------------------Right Goal--------------------------------------------------------------		
		elsif	(Hcount <= center_gxstart_r + HBACK_PORCH + HSYNC and Vcount <= center_gystart + VBACK_PORCH + VSYNC )then
			
			if (center_gxstart_r + HBACK_PORCH + HSYNC - Hcount) <=gradius and (center_gystart+VBACK_PORCH+VSYNC-Vcount) <= gradius
			and (table(TO_INTEGER(center_gxstart_r - Hcount)+ HBACK_PORCH + HSYNC) + table(TO_INTEGER(center_gystart-Vcount)+VBACK_PORCH+VSYNC))<= (sqr_gradius) then
				goal_outer_circle <= '1';
			else
				goal_outer_circle <= '0';
			end if;
		elsif	(Hcount <= center_gxstart_r+HBACK_PORCH+HSYNC and Vcount >= center_gystart+VBACK_PORCH+VSYNC) then	 
			
			if (center_gxstart_r + HBACK_PORCH + HSYNC - Hcount) <=gradius and (Vcount-center_gystart-VBACK_PORCH-VSYNC) <= gradius
			and (table(TO_INTEGER(center_gxstart_r - Hcount)+ HBACK_PORCH + HSYNC) + table(TO_INTEGER(Vcount-center_gystart)-VBACK_PORCH-VSYNC))<= (sqr_gradius) then
				goal_outer_circle <= '1';
			else
				goal_outer_circle<='0';
			end if;
		else
			goal_outer_circle <= '0';
		end if;
	end if;
  end process;
-----------------------------------------goal inner semi circle generation----------------------------------------------------------
goalgeninner: process(clk25)
begin
	if rising_edge (clk25) then
		if reset = '1' then
			goal_inner_circle <= '0';
elsif	(Hcount>= center_gxstart_l + HBACK_PORCH+HSYNC and Vcount >= center_gystart+VBACK_PORCH+VSYNC
					and Hcount <= center_gxstart_l + gradius + HBACK_PORCH + HSYNC) then		
     if (Hcount-center_gxstart_l - HBACK_PORCH - HSYNC) <=gradius_inner and (Vcount-center_gystart-VBACK_PORCH-VSYNC) <=gradius_inner 
	  and (table(TO_INTEGER(Hcount-center_gxstart_l)-HBACK_PORCH-HSYNC) + table(TO_INTEGER(Vcount-center_gystart)-VBACK_PORCH-VSYNC))<= (sqr_gradius_inner) then
					goal_inner_circle <= '1';
			else
					goal_inner_circle <= '0';
			end if;		
elsif	(Hcount>= center_gxstart_l + HBACK_PORCH + HSYNC and Vcount <= center_gystart + VBACK_PORCH + VSYNC
					and Hcount <= center_gxstart_l + gradius + HBACK_PORCH + HSYNC) then  
	if (Hcount-center_gxstart_l - HBACK_PORCH - HSYNC) <=gradius_inner and (center_gystart+VBACK_PORCH+VSYNC-Vcount) <=gradius_inner 
	  and (table(TO_INTEGER(Hcount-center_gxstart_l)-HBACK_PORCH-HSYNC) + table(TO_INTEGER(center_gystart-Vcount)+VBACK_PORCH+VSYNC))<= (sqr_gradius_inner) then
					goal_inner_circle <= '1'; 
			else
					goal_inner_circle <= '0';
			end if;
elsif	(Hcount <= center_gxstart_r + HBACK_PORCH + HSYNC and Vcount <= center_gystart + VBACK_PORCH + VSYNC )then
	if (center_gxstart_r + HBACK_PORCH + HSYNC - Hcount) <=gradius_inner and (center_gystart+VBACK_PORCH+VSYNC-Vcount) <=gradius_inner 
	and (table(TO_INTEGER(center_gxstart_r - Hcount)+ HBACK_PORCH + HSYNC )+ table(TO_INTEGER(center_gystart-Vcount)+VBACK_PORCH+VSYNC))<= (sqr_gradius_inner) then
				goal_inner_circle <= '1';
			else
				goal_inner_circle <= '0';
			end if;
			
elsif	(Hcount <= center_gxstart_r+HBACK_PORCH+HSYNC and Vcount >= center_gystart+VBACK_PORCH+VSYNC) then	 
	if (center_gxstart_r + HBACK_PORCH + HSYNC - Hcount) <=gradius_inner and (Vcount-center_gystart-VBACK_PORCH-VSYNC)<=gradius_inner 
	and (table(TO_INTEGER(center_gxstart_r - Hcount)+ HBACK_PORCH + HSYNC )+ table(TO_INTEGER(Vcount-center_gystart)-VBACK_PORCH-VSYNC))<= (sqr_gradius_inner) then
				goal_inner_circle <= '1';
			else
				goal_inner_circle<='0';
			end if;							
else
			goal_inner_circle <= '0';
		end if;
	end if;
  end process;
----------------------------------goal generation--------------------------------------------------------------
Goalgeneration : process(clk25)
  begin
    if rising_edge (clk25) then
        if reset = '1' then
            goal <= '0';
        elsif (Vcount >= goal_lower_slot+VSYNC+VBACK_PORCH and Vcount <= goal_upper_slot+ VSYNC+VBACK_PORCH ) and ((Hcount=HSYNC+HBACK_PORCH) or (Hcount= HTOTAL-HFRONT_PORCH-1)) then
            goal <= '1';
        else
            goal <= '0';
        end if;
    end if;
  end process; 
----------------------------------------------------------------------------------------------------------------						
Scoregenplayer1	: process(clk25)
variable htmp: unsigned (9 downto 0);
variable vtmp: unsigned (9 downto 0);
variable ctmp: unsigned (0 to 50) ;	
	
	begin
		if rising_edge(clk25) then
			if reset =  '1' then
				score_player1 <= '0';
			elsif (((Hcount >= HBACK_PORCH + HSYNC and Hcount < 52 + HBACK_PORCH + HSYNC) )
				and (Vcount > 468 + VSYNC+ VBACK_PORCH  and Vcount <= 476 +  VSYNC+ VBACK_PORCH)) then
				
				   htmp := Hcount - HSYNC - HBACK_PORCH -0 ;
				   vtmp := Vcount - VSYNC - VBACK_PORCH - 469 ;
				   ctmp := ( others => '0' ) ;
				   ctmp := crange(TO_INTEGER(vtmp)) ;
				   if ctmp(TO_INTEGER( htmp ) ) = '1' then
				   score_player1 <= '1' ;
				   elsif ctmp(TO_INTEGER(htmp)) = '0' then
				   score_player1 <= '0' ;
				   end if ;
			else
				score_player1 <= '0';
			end if;
		end if;
	end process;
--------------------------------------------------Player2--------------------------------------------
Scoregenplayer2	: process(clk25)
variable htmp1: unsigned (9 downto 0);
variable vtmp1: unsigned (9 downto 0);
variable ctmp1: unsigned (0 to 54) ;	
	
	begin
		if rising_edge(clk25) then
			if reset =  '1' then
				score_player2 <= '0';
			elsif (((Hcount >= HBACK_PORCH + HSYNC +500 and Hcount < 556 + HBACK_PORCH + HSYNC) )
				and (Vcount > 468 + VSYNC+ VBACK_PORCH  and Vcount <= 476 +  VSYNC+ VBACK_PORCH)) then
				
				   htmp1 := Hcount - HSYNC - HBACK_PORCH - 500 ;
				   vtmp1 := Vcount - VSYNC - VBACK_PORCH - 469 ;
				   ctmp1 := ( others => '0' ) ;
				   ctmp1 := crange1(TO_INTEGER(vtmp1)) ;
				   if ctmp1(TO_INTEGER( htmp1 ) ) = '1' then
				   score_player2 <= '1' ;
				   elsif ctmp1(TO_INTEGER(htmp1)) = '0' then
				   score_player2 <= '0' ;
				   end if ;
			else
				score_player2 <= '0';
			end if;
		end if;
	end process;
-----------------------------------------------------------------------------------------------------
-----------------------------------------socre keepig for player 1 ----------------------------------
PLAYER1_score	: process(clk25)
variable htmp1: unsigned (9 downto 0);
variable vtmp1: unsigned (9 downto 0);
variable ctmp1: unsigned (0 to 71) ;	
								
	begin
		if rising_edge(clk25) then
			
			if reset =  '1' then
				sc_player_1 <= '0';
			elsif (((Hcount >= HBACK_PORCH + HSYNC +56 and Hcount < 63 + HBACK_PORCH + HSYNC) )
				and (Vcount > 468 + VSYNC+ VBACK_PORCH  and Vcount <= 476 +  VSYNC+ VBACK_PORCH)) then
				   htmp1 := Hcount - HSYNC - HBACK_PORCH - 56 ;
				   vtmp1 := Vcount - VSYNC - VBACK_PORCH - 469 ;
				   ctmp1 := ( others => '0' ) ;
				   ctmp1 := crange2(TO_INTEGER(vtmp1)) ;
				   if ctmp1((TO_INTEGER( htmp1 ) )+ (player_1 * 8) ) = '1' then 
				   sc_player_1 <= '1' ;
				   elsif ctmp1((TO_INTEGER(htmp1))+ (player_1 * 8)) = '0' then 
				   sc_player_1 <= '0' ;
				   end if ;
			else
				sc_player_1 <= '0';
			end if;
		end if;
	end process;
	
---------------------------------------------------------------------------------------------------------
-----------------------------------------------score keeping for player 2----------------------------
PLAYER2_score	: process(clk25)
variable htmp1: unsigned (9 downto 0);
variable vtmp1: unsigned (9 downto 0);
variable ctmp1: unsigned (0 to 71) ;	
								
	begin
		if rising_edge(clk25) then
			
			if reset =  '1' then
				sc_player_2 <= '0';
			elsif (((Hcount >= HBACK_PORCH + HSYNC + 558 and Hcount < 565 + HBACK_PORCH + HSYNC) )
				and (Vcount > 468 + VSYNC+ VBACK_PORCH  and Vcount <= 476 +  VSYNC+ VBACK_PORCH)) then
				   htmp1 := Hcount - HSYNC - HBACK_PORCH - 558 ;
				   vtmp1 := Vcount - VSYNC - VBACK_PORCH - 469 ;
				   ctmp1 := ( others => '0' ) ;
				   ctmp1 := crange2(TO_INTEGER(vtmp1)) ;
				   if ctmp1((TO_INTEGER( htmp1 ) )+ (player_2 * 8 ) ) = '1' then 
				   sc_player_2 <= '1' ;
				   elsif ctmp1((TO_INTEGER(htmp1))+ (player_2 * 8 )) = '0' then
				   sc_player_2 <= '0' ;
				   end if ;
			else
				sc_player_2 <= '0';
			end if;
		end if;
	end process;
-----------------------------------------------------------------------------------------------------
VideoOut: process (clk25, reset_n)
  begin
    if reset = '1' then
      VGA_R <= "0000000000";
      VGA_G <= "0000000000";
      VGA_B <= "0000000000";
    elsif rising_edge (clk25) then
        if goal= '1' then
		VGA_R <= "1111111111";
        VGA_G <= "0000000000";
        VGA_B <= "0000000000"; 
		elsif puck_circle = '1' then
		VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "1111111111";
		elsif circle ='1' or ethernet_circle ='1' then
		VGA_R <= "1111111111";
        VGA_G <= "1111111111";
        VGA_B <= "1111111111";
        elsif goal_inner_circle = '1' then    --- maintaining the hierarchichal order of gen the graphics.
		VGA_R <= "0000000000";
        VGA_G <= "1111100111";
        VGA_B <= "1111111111";
		elsif vline = '1' or goal_outer_circle = '1' then
		VGA_R <= "1111111111";
        VGA_G <= "1111111111";
        VGA_B <= "1111111111";		
		elsif small_center_circle = '1' then   --- maintaining the hierarchichal order of gen the graphics.
		VGA_R <= "0000000000";
        VGA_G <= "1111100111";
        VGA_B <= "1111111111"; 
		elsif center_circle = '1' or score_player1 =  '1' or sc_player_1 = '1' or score_player2 = '1' or sc_player_2 = '1' then
        VGA_R <= "1111111111";
        VGA_G <= "1111111111";
        VGA_B <= "1111111111";
	  elsif ((vga_hblank = '0' and vga_vblank ='0')and Vcount <= 464 +VSYNC+VBACK_PORCH) then  -- to leave space in the end for player name and score.
        VGA_R <= "0000000000";
        VGA_G <= "1111100111";
        VGA_B <= "1111111111";
	  else
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "0000000000";    
      end if;
    end if;
  end process VideoOut;

  VGA_CLK <= clk25;
  VGA_HS <= not vga_hsync;
  VGA_VS <= not vga_vsync;
  VGA_SYNC <= '0';
  VGA_BLANK <= not (vga_hsync or vga_vsync);

end rtl;
